package com.ruoyi.inventory.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.ruoyi.asset.domain.TbAssetInformation;
import com.ruoyi.asset.mapper.TbAssetInformationMapper;
import com.ruoyi.inventory.domain.TagInfo;
import com.ruoyi.inventory.domain.dto.TakeStockDTO;
import com.ruoyi.inventory.domain.dto.TbAssetInventoryDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.inventory.mapper.TbAssetInventoryMapper;
import com.ruoyi.inventory.domain.TbAssetInventory;
import com.ruoyi.inventory.service.ITbAssetInventoryService;

/**
 * 资产盘点记录Service业务层处理
 * 
 * @author ydl
 * @date 2023-06-01
 */
@Service
public class TbAssetInventoryServiceImpl implements ITbAssetInventoryService
{
    @Autowired
    private TbAssetInventoryMapper tbAssetInventoryMapper;

    @Autowired
    private TbAssetInformationMapper informationMapper;

    /**
     * 查询资产盘点记录
     * 
     * @param id 资产盘点记录主键
     * @return 资产盘点记录
     */
    @Override
    public TbAssetInventory selectTbAssetInventoryById(Long id)
    {
        return tbAssetInventoryMapper.selectTbAssetInventoryById(id);
    }

    /**
     * 查询资产盘点记录列表
     * 
     * @param tbAssetInventory 资产盘点记录
     * @return 资产盘点记录
     */
    @Override
    public List<TbAssetInventory> selectTbAssetInventoryList(TbAssetInventory tbAssetInventory)
    {
        return tbAssetInventoryMapper.selectTbAssetInventoryList(tbAssetInventory);
    }

    /**
     * 新增资产盘点记录
     * 
     * @param tbAssetInventory 资产盘点记录
     * @return 结果
     */
    @Override
    public int insertTbAssetInventory(TbAssetInventory tbAssetInventory)
    {
        return tbAssetInventoryMapper.insertTbAssetInventory(tbAssetInventory);
    }

    @Override
    public int addByTakeStock(TakeStockDTO dto){
        //解析实体
        List<TagInfo> list = dto.getAssetList();
        String location = dto.getInventoryLocation();
        String orderNumber = dto.getOrderNumber();
        //获取所在位置内的资产
        TbAssetInformation information = new TbAssetInformation();
        information.setLocationNumber(location);
        List<TbAssetInformation> informations = informationMapper.selectTbAssetInformationList(information);
        //对比盘点
        List<TbAssetInformation> intersection = new ArrayList<>(informations);
        intersection.retainAll(list);
        if (intersection.size()<informations.size()){
            //盘亏
            if (!orderNumber.isEmpty()){
                TbAssetInventory inventory = tbAssetInventoryMapper.selectTbAssetInventoryByNumber(orderNumber);
                inventory.setInventoryStatus(2L);
                inventory.setInventoryResult(1L);
                return updateTbAssetInventory(inventory);
            }else {
                TbAssetInventory inventory = new TbAssetInventory();
                inventory.setInventoryStatus(2L);
                inventory.setInventoryResult(1L);
                inventory.setInventoryLocation(location);
                inventory.setInventoryDate(new Date());
                inventory.setName("线下自测");
                return insertTbAssetInventory(inventory);
            }
        }else if (list.size()>intersection.size()){
            //盘盈
            if (!orderNumber.isEmpty()){
                TbAssetInventory inventory = tbAssetInventoryMapper.selectTbAssetInventoryByNumber(orderNumber);
                inventory.setInventoryStatus(2L);
                inventory.setInventoryResult(2L);
                return updateTbAssetInventory(inventory);
            }else {
                TbAssetInventory inventory = new TbAssetInventory();
                inventory.setInventoryStatus(2L);
                inventory.setInventoryResult(2L);
                inventory.setInventoryLocation(location);
                inventory.setInventoryDate(new Date());
                inventory.setName("线下自测");
                return insertTbAssetInventory(inventory);
            }
        }else if (intersection.size()==informations.size()){
            //正常
            if (!orderNumber.isEmpty()){
                TbAssetInventory inventory = tbAssetInventoryMapper.selectTbAssetInventoryByNumber(orderNumber);
                inventory.setInventoryStatus(2L);
                inventory.setInventoryResult(3L);
                return updateTbAssetInventory(inventory);
            }else {
                TbAssetInventory inventory = new TbAssetInventory();
                inventory.setInventoryStatus(2L);
                inventory.setInventoryResult(3L);
                inventory.setInventoryLocation(location);
                inventory.setInventoryDate(new Date());
                inventory.setName("线下自测");
                return insertTbAssetInventory(inventory);
            }
        }else {
            return 0;
        }
    }

    /**
     * 修改资产盘点记录
     * 
     * @param tbAssetInventory 资产盘点记录
     * @return 结果
     */
    @Override
    public int updateTbAssetInventory(TbAssetInventory tbAssetInventory)
    {
        return tbAssetInventoryMapper.updateTbAssetInventory(tbAssetInventory);
    }

    /**
     * 批量删除资产盘点记录
     * 
     * @param ids 需要删除的资产盘点记录主键
     * @return 结果
     */
    @Override
    public int deleteTbAssetInventoryByIds(Long[] ids)
    {
        return tbAssetInventoryMapper.deleteTbAssetInventoryByIds(ids);
    }

    /**
     * 删除资产盘点记录信息
     * 
     * @param id 资产盘点记录主键
     * @return 结果
     */
    @Override
    public int deleteTbAssetInventoryById(Long id)
    {
        return tbAssetInventoryMapper.deleteTbAssetInventoryById(id);
    }

    @Override
    public List<TbAssetInventoryDTO> selectTbAssetInventoryDTOList(TbAssetInventoryDTO dto) {
        return tbAssetInventoryMapper.selectTbAssetInventoryDTOList(dto);
    }
}
